/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Martin Rexa
 */

package cz.xelfi.quoridor.statistics;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.webidor.Game;
import cz.xelfi.quoridor.webidor.CommentedMove;
import cz.xelfi.quoridor.IllegalPositionException;
import java.util.Map;
import java.util.HashMap;

/**
 *
 * @author Martin Rexa
 */
public class OpeningTree {
    Map<String, OpeningTreeNode> nodes = new HashMap<String, OpeningTreeNode>();
    OpeningTreeNode root;

    public OpeningTree(){
        root = new OpeningTreeNode("ROOT");
        nodes.put("ROOT", root);
    }

    public OpeningTreeNode getRoot(){
        return root;
    }

    public OpeningTreeNode getNode(String hashCode){
        return nodes.get(hashCode);
    }

    public void processGame(Game game) throws IllegalPositionException{
        OpeningTreeNode node = root;
        OpeningTreeNode parentNode;
        OpeningTreeNode mirrorNode = root;
        OpeningTreeNode mirrorParentNode;
        Board board = Board.empty();
        Board mirrorBoard = Board.empty();
        root.addGame(game);
        for(CommentedMove move: game.getMoves()){
            if(!move.getMove().equals(Move.RESIGN)){
                parentNode = node;
                mirrorParentNode = mirrorNode;
                board = board.apply(move.getMove());
                mirrorBoard = mirrorBoard.apply(move.getMove().getMirrorMove());
                String hashCode = board.getCode();
                String mirrorHashCode = mirrorBoard.getCode();
                node = nodes.get(hashCode);
                if(node == null){
                    node = new OpeningTreeNode(hashCode);
                    nodes.put(hashCode, node);
                }
                node.addGame(game);
                mirrorNode = nodes.get(mirrorHashCode);
                if(mirrorHashCode.equals(hashCode)){
                    if(mirrorNode == null){
                        mirrorNode = node;
                    }
                }else{
                    if(mirrorNode == null){
                        mirrorNode = new OpeningTreeNode(mirrorHashCode);
                        nodes.put(mirrorHashCode, mirrorNode);
                    }
                    mirrorNode.addGame(game);
                }
                if(parentNode != null)
                    parentNode.addChild(move.getMove(),node);
                if(mirrorParentNode != null){
                    if(parentNode.equals(mirrorParentNode) && node.equals(mirrorNode))
                        ;
                    else
                        mirrorParentNode.addChild(move.getMove().getMirrorMove(),mirrorNode);
                }
            }
        }
        node.addFinishedGame(game);
        if(!node.equals(mirrorNode))
            mirrorNode.addFinishedGame(game);
    }

}
